/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  Mafia II frameresource.bin definitions
 *--------------------------------------------------------------------------
 * @histroy: 19.09.2010. Created
 * @histroy: 07.12.2012. Remastering for ZModeler3
 */
#ifndef mafiaframeresource_h_included
#define mafiaframeresource_h_included
#include <zmodeler.h>
#include "MafiaCommon.h"
#include "FrameResourceScene.h"
#include "FrameResourceCollision.h"

namespace core
{
  namespace io
  {

#pragma pack (push, 1)
    struct  tRootGroupInformation
    {
      long      m_nZero;
      float     m_fDistance; //? lod?
      long      m_nZero2[2];//0
      WORD      m_nPad;//0
      float     m_fSetup[12];//always the same;
      BYTE      m_bUnk; //0?
      tHashNamePackSeq m_rootGroops;
    };

    //------------------------------------
    //    space partition information
    //------------------------------------

    struct tSpacePartition
    {
      DWORD       m_dwMemRequireA;  //0x4-aligned memory requirement for blocks: partition+set#1+set#2
      DWORD       m_dwMemRequireB;  //==m_dwMemRequireA;
      long        m_nPartitionType; // 3 or 4?
      BYTE        m_bAvailA;        // 1 = inplace array follows; 0 = block missing
      //-- partition planes/volumes (avail if m_bAvailA != 0)
      float           m_fUnk3[6];       //division values for conversion "short int" -> "float" in m_seqDesc1 array?
      long            m_numDesc1;
      tDescriptorSeq  m_seqDesc1;   //20 bytes each;
      //-- end of partition planes/volumes

      //-- partition set #1 block
      long        m_numLongs1;  //looks like == (m_numDesc1 + 1)
      BYTE        m_bAvailB;    
      tLongSeq    m_seqLongs1;  //unknown. sorted on increase, covers range more than vertices pool size.
      //-- end partition set #1 block

      //-- faces sorting set #2 block
      long        m_numLongs2;  //matches amount of polygons;
      BYTE        m_bAvailC;
      tLongSeq    m_seqLongs2;  //unknown. values ranges [0..m_numLongs2]. something like pre-sorted mapping?
      //-- end of faces sorting set #2 block
    };

    //------------------------------------
    //    material bursts information
    //------------------------------------
    struct tMaterialBurst
    {
      short       m_bounds[6]; //min x,y,z, max x,y,z; no short2float conversion needed! pure cast/truncate;
      WORD        m_firstIndex;
      WORD        m_lastIndex;
      short       m_nNeighbourIndex1; //-1 if none; refers to burst index;
      short       m_nNeighbourIndex2; //-1 if none; refers to burst index;
    };
    typedef ZSequence<tMaterialBurst> tMaterialBurstSeq;
    struct tMaterialSplit
    {
      long        m_nFirstBurst;
      long        m_numBurst;
      long        m_nBaseIndex;   //in associated index buffer;
    };
    typedef ZSequence<tMaterialSplit> tMaterialSplitSeq;

    struct tMaterialSpltInfo
    {
      long        m_nUnk18;     //0x2 version?
      long        m_numVerts;   //amount of vertices
      long        m_numFaces;   //amount of polygons

      long        m_nUnk20;     //0
      long        m_nUnk21;     //0xC type?
      long        m_numSplitGroup; //1 or 0 (amount of split gropus;
      //{ begin split information group (missing if m_numSplitGroup==0
      BYTE        m_bAvailD;    
      long        m_nUnk24;     //1
      long        m_nSizeOfMatBurstEntries;   //always 0x14: sieof(tMaterialBurst)
      long        m_nSizeOfMatSplitEntries;   //always 0xC: sizeof(tMaterialSplit)
      long        m_numMatBurst; //
      long        m_numMatSplit;// amount of material splits in indices buffer;
      QWORD       m_dwHash29;

      tMaterialBurstSeq  m_seqMatBurst;
      tMaterialSplitSeq  m_seqMatSplit;
      //} end of split information group;
    };

    //--------------------------------
    //   Geometry LOD fragment:
    //--------------------------------
    struct tGeometryLOD
    {
      float         m_fUnk2;  //some float value; visibility range?
      tHashNamePack m_nameIB;
      DWORD         m_dwVertsInfo; //0x1003D5: flags or vertex format?
      tHashNamePack m_nameVB;
      DWORD         m_numVertsInBuf;
      long          m_nZero1; // 0?
      long          m_nPartition;     // 1 or 0 (if not available)

      tSpacePartition partition;    // partition is available if m_nPartition != 0;

      long          m_nMatSplitType;  // 0xC = standard; 0x0 = not avail?
      tMaterialSpltInfo m_matSplit; // avail if m_nMatSplitType != 0

      long          m_nZeroTail;      // should be 0;
    };

    typedef ZSequence<tGeometryLOD>  tGeometryLODSeq;

    //--------------------------------
    //   Geometry model definition:
    //--------------------------------
    struct tGeometryModelDefinition
    {
      //--- geometry definition block;
      BYTE        m_nLODs;  //num of LODs?
      WORD        m_nZero;  //0 on cars; can be non-zero on town models;
      float       m_fBounds[4];

      tGeometryLODSeq m_LODs;
    };
    typedef ZSequence<tGeometryModelDefinition> tGeometryModelSeq;

    struct  tFrameResource
    {
      BYTE        m_bGrouping;        // set 0 if no grouping;
      long        m_nRootGroups;      // amount of root groups;
      long        m_nGeomDefinitions; 
      long        m_nModelUnkDefs;    
      long        m_nSceneCompositions; //can be 0 on scene models
      long        m_nHierarchySets;     //can be 0 on scene models;
      long        m_nHierarchyParenting;//can be 0 on scene models;
      long        m_nCollisionBlocks;
      //--- if grouping available (m_bGroupint != 0) group info follows:
      tRootGroupInformation     m_grouping;

      tGeometryModelSeq         m_models;

      tLODDescriptorSeq         m_modelUnks;//collision or visibility partition? length == m_nModelUnkDefs

      tSceneCompositionSetSeq   m_sceneSets;    //amount == m_nSceneCompositions;
      tHierarchyAssemblySeq     m_hierarchy;    //amount == m_nHierarchySets;
      tHierarchyParentingSeq    m_setParenting; //amount == m_nHierarchyParenting;
      tCollisionPtrSeq          m_collision;
    };

#pragma pack (pop)

  };//io
};//core


#endif //#mafiaframeresource_h_included

