/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  Mafia II frameresource.bin definitions
 *--------------------------------------------------------------------------
 * @histroy: 19.09.2010. Created
 * @histroy: 07.12.2012. Remastering for ZModeler3
 */
#ifndef mafiaframeresource_collision_h_included
#define mafiaframeresource_collision_h_included
#include <zmodeler.h>
#include <core/io/IStream.h>
#include "MafiaCommon.h"

namespace core
{
  namespace io
  {

#pragma pack (push, 1)
    class tCollisionBase
    {
    public:
      virtual ~tCollisionBase() {}
      virtual ZRESULT readFromStream(core::io::IStream* pStream, long numBones);

      tHashNamePack   m_name;
      DWORD           m_dwFlags;  //flags; usually 0x01;
      tRowMajorMarix  m_mat;
      short           m_nBase1;   //always 0xFFFF;
      long            m_nBaseV1;  //can be -1
      long            m_nBaseV2;  //can be -1;
      short           m_nBase2;   //always 0xFFFF; 
    };

    class tCollisionType0 : public tCollisionBase
    {
    public:
      virtual ~tCollisionType0() {}
      virtual ZRESULT readFromStream(core::io::IStream* pStream, long numBones);

      BYTE    m_bZero;    //always 0;
    };

    // unknown blocks stored in type11:
    struct tCollUnk1
    {
      BYTE    m_num;
      long    m_value;
    };
    typedef ZSequence<tCollUnk1>  tCollUnk1Seq;

    class tCollisionType1 : public tCollisionType0
    {
    public:
      virtual ~tCollisionType1() {}
      virtual ZRESULT readFromStream(core::io::IStream* pStream, long numBones);

      DWORD         m_dwUnk1; //0x28000070?
      tBound        m_bound;
      BYTE          m_FF;     //always 0xFF;
      long          m_nGeomIndex;// geometry index (can be shared/instanced)
      long          m_nValue2;//?
      tHashNamePack m_name1;
      WORD          m_wUnk3;  //usually 0
      BYTE          m_controlByte; //0 on type1; can be different on type x11;
    };

    class tCollisionType11 : public tCollisionType1
    {
    public:
      virtual ~tCollisionType11() {}
      virtual ZRESULT readFromStream(core::io::IStream* pStream, long numBones);

      // m_controlByte is non-zero;
      long              m_nVal[3];  //usually 6,7,8 or 4,5,6;

      tRowMajorMarixSeq m_mats;     //matrices: amount: num bones in scene + 1;

      long              m_num1, m_num2;
      tCollUnk1Seq      m_seqUnk1;
      // probably, volumetric data? polygons and vertices?
      long              m_sizePart1;
      long              m_sizePart2;
      tBYTESeq          m_dataPart1;
      tBYTESeq          m_dataPart2;  //each entry is 0x10 bytes;
    };

    class tCollisionType2 : public tCollisionType0
    {
    public:
      virtual ~tCollisionType2() {}
      virtual ZRESULT readFromStream(core::io::IStream* pStream, long numBones);

      tHashNamePack m_name2;
    };

    class tCollisionType7 : public tCollisionType0
    {
    public:
      virtual ~tCollisionType7() {}
      virtual ZRESULT readFromStream(core::io::IStream* pStream, long numBones);

      tBound        m_bound7;
    };

    class tCollisionType1D8 : public tCollisionBase
    {
    public:
      virtual ~tCollisionType1D8() {}
      virtual ZRESULT readFromStream(core::io::IStream* pStream, long numBones);

      QWORD         m_hashUnk;
    };

    typedef ZSequence<ZStdOwnPtr<tCollisionBase> >  tCollisionPtrSeq;
#pragma pack (pop)

  };//io
};//core


#endif //mafiaframeresource_collision_h_included

