/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  Mafia II frameresource.bin definitions
 *--------------------------------------------------------------------------
 * @histroy: 19.09.2010. Created
 * @histroy: 07.12.2012. Remastering for ZModeler3
 */
#ifndef mafiaframeresource_scene_h_included
#define mafiaframeresource_scene_h_included
#include <zmodeler.h>
#include "MafiaCommon.h"

namespace core
{
  namespace io
  {

#pragma pack (push, 1)
    struct tSceneCompose1
    {
      tRowMajorMarix  m_mat4x3;  //row-major matrix pack;
      tBound          m_fBounds; //
      BYTE            m_nUnk;    //always 0xFF = info is valid; 0x00 = info is empty/zero;
    };
    typedef ZSequence<tSceneCompose1> tSceneComposeSeq;
    struct tSceneCompositionInfo
    {
      long                m_nIDs;
      long                m_nNums;      //?
      BYTE                m_bytes[4];    // [0] + [1]  should match m_nIDs.
      long                m_nZero;
      tBYTESeq            m_IDs;        // amount == m_nBlocks;
      tUShortSeq          m_nums;       // flags?
    };
    typedef ZSequence<tSceneCompositionInfo>  tSceneCompositionInfoSeq;
    struct tSceneCompositionSet
    {
      long                m_nBlocks;    //bones?
      BYTE                m_nSet;       //LODs?
      tSceneCompositionInfoSeq  m_seq;  //amount == m_nSet;
      tSceneComposeSeq    m_blocks;     //amount == m_nBlocks
      tBound              m_bounds;     //can be left uninitialized 0x7f7fffff on min; 0xff7fffff on max;
    };
    typedef ZSequence<tSceneCompositionSet> tSceneCompositionSetSeq;

    struct tHierarchyParenting
    {
      long      m_num;            //usually matches amount of nodes;
      tBYTESeq  m_seqParents;     //[m_numRefs]; 0xFF == root;
      //---------- below is a considerable set;
      BYTE      n1,n2;            //0,1 or 0,3? or a part of following set(s)???
      tBYTESeq  m_set1;           //[m_numRefs]; value can reach m_num?
      tBYTESeq  m_set2;           //[m_numRefs];
    };
    typedef ZSequence<tHierarchyParenting>  tHierarchyParentingSeq;
    struct tUnknownHierarchySet
    {
      tBoundSeq m_seqBounds;    //amount of entries == m_numA2? or numA[2];
      //---------- below is a considerable set.
      tBYTESeq  m_seqBytes1;    //amount is numA[2];
      BYTE      m_nBytes;       //may be this is a part of m_setBytes1?
      tBYTESeq  m_seqBytes2;    //amont is m_numB1;
    };
    typedef ZSequence<tUnknownHierarchySet> tUnknownHierarchySetSeq;
    struct tHierarchyAssembly
    {
      long      m_numA[4];          //usually all the same?
      long      m_numB;
      long      m_numUnkSet;
      tLongSeq  m_numInUnkSet;      //amount == m_numUnkSet;
      BYTE      m_idType;           //1 == ident by hash (numeric value); 3 == ident by name or hash (full 8 byte hash)
      tHashNamePackSeq  m_nodes;    //amount == numA[0];
      tRowMajorMarixSeq m_nodeMats; //amount == numA[1];
      long      m_numA2;            //==m_numA[0] ? 
      tBYTESeq  m_jointTypes;       // amount == m_numA2? or numA[2];
      tRowMajorMarixSeq m_nodeMats2;//amount == m_numA2? or numA[2];

      tUnknownHierarchySetSeq m_setUnk; //amount == m_numUnkSet;
    };
    typedef ZSequence<tHierarchyAssembly> tHierarchyAssemblySeq;

    struct tDescriptor
    {
      long        m_n1;
      long        m_n2;
      short       m_w[6];
    };
    typedef ZSequence<tDescriptor>    tDescriptorSeq;
    struct tMaterialAssignFragment
    {
      long        m_numTris;
      long        m_nFirstIndex;
      QWORD       m_matHash;
      DWORD       m_dwZero;
    };
    typedef ZSequence<tMaterialAssignFragment>    tMaterialAssignSeq;
    struct tLODDescriptor
    {
      BYTE                          m_nLods;
      // followed by long[m_nLods] -- amount of descs in respective lod;
      float                         m_fBounds[6]; //tBound? shared among lods;
      ZSequence<tMaterialAssignSeq> m_assign;     //one assginemnt set per each lod;
    };
    typedef ZSequence<tLODDescriptor> tLODDescriptorSeq;

#pragma pack (pop)

  };//io
};//core


#endif //mafiaframeresource_scene_h_included

