/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  MafiaII collision data (frame resource fragment) import;
 *--------------------------------------------------------------------------
 * @histroy: 28.12.2012. Created
 */
#include "MafiaII.h" 


namespace core
{

namespace io
{

//-------------------------------------------------------
// @name : tCollisionBase::readFromStream
// reads collision basic block information;
// @param core::io::IStream* pStream     : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT tCollisionBase::readFromStream(core::io::IStream* pStream, long numBones)
{
  readHashLenString(&m_name, pStream);
  pStream->read(&m_dwFlags, 4);
  pStream->read(&m_mat, 0x30);
  pStream->read(&m_nBase1, 12);

  DEBUG_MESSAGE4("CollBase: %s, %li %li, Flags: 0x%X\n", m_name.m_strName.length() ? (LPCTSTR)m_name.m_strName : "unnamed",
                 m_nBaseV1, m_nBaseV2, m_dwFlags);
  return ZRESULT_OK;
}

ZRESULT tCollisionType0::readFromStream(core::io::IStream* pStream, long numBones)
{
  tCollisionBase::readFromStream(pStream, numBones);
  pStream->read(&m_bZero, 1);
  return ZRESULT_OK;
}

ZRESULT tCollisionType1::readFromStream(core::io::IStream* pStream, long numBones)
{
  tCollisionType0::readFromStream(pStream, numBones);
  pStream->read(&m_dwUnk1, 4+sizeof(tBound)+1+4+4);
  readHashLenString(&m_name1, pStream);
  pStream->read(&m_wUnk3, 2+1);
  DEBUG_MESSAGE4("  type1, values: {%li, %li}, 0x%hx, control byte: %li\n", m_nGeomIndex, m_nValue2, m_wUnk3, (long)m_controlByte);
  return ZRESULT_OK;
}



//-------------------------------------------------------
// @name : tCollisionType11::readFromStream
// Collision block of type 11; pretty unknown stuff; 
// @param core::io::IStream* pStream     : 
//
// @return ZRESULT  : 
//-------------------------------------------------------
ZRESULT tCollisionType11::readFromStream(core::io::IStream* pStream, long numBones)
{
  tCollisionType1::readFromStream(pStream, numBones);
  pStream->read(m_nVal, 12);

  m_mats.length(numBones+1);
  pStream->read(m_mats.getBuffer(), 0x30*(numBones+1));
  pStream->read(&m_num1, 8);
  DEBUG_MESSAGE3("  type11, num: %li, %li (bones: %li)\n", m_num1, m_num2, numBones);

  // unknown blocks;
  m_seqUnk1.length(m_num1);
  pStream->read(m_seqUnk1.getBuffer(), m_num1*5);
  pStream->read(&m_sizePart1, 8);
  m_dataPart1.length(m_sizePart1);
  m_dataPart2.length(m_sizePart2);
  pStream->read(m_dataPart1.getBuffer(), m_sizePart1);
  pStream->read(m_dataPart2.getBuffer(), m_sizePart2);

  return ZRESULT_OK;
}

ZRESULT tCollisionType2::readFromStream(core::io::IStream* pStream, long numBones)
{
  tCollisionType0::readFromStream(pStream, numBones);
  readHashLenString(&m_name2, pStream);
  return ZRESULT_OK;
}

ZRESULT tCollisionType7::readFromStream(core::io::IStream* pStream, long numBones)
{
  tCollisionType0::readFromStream(pStream, numBones);
  pStream->read(&m_bound7, sizeof(tBound));
  return ZRESULT_OK;
}

ZRESULT tCollisionType1D8::readFromStream(core::io::IStream* pStream, long numBones)
{
  tCollisionBase::readFromStream(pStream, numBones);
  pStream->read(&m_hashUnk, 8);
  return ZRESULT_OK;
}



//-------------------------------------------------------
// @name : CMafiaII::readCollisionBlocks
// Reads collision blocks for frame resource; 
// @param tFrameResource& frame          : 
// @param core::io::IStream* pStream     : 
//
// @return ZRESULT  : 
//-------------------------------------------------------
ZRESULT CMafiaII::readCollisionBlocks(tFrameResource& frame, core::io::IStream* pStream)
{
  if (0 == frame.m_nCollisionBlocks)
    return ZRESULT_FALSE;//no collision data available;
  frame.m_collision.length(frame.m_nCollisionBlocks);
  tLongSeq seqCollTypes;
  seqCollTypes.length(frame.m_nCollisionBlocks);
  pStream->read(seqCollTypes.getBuffer(), 4*frame.m_nCollisionBlocks);
  //
  // create and read each collision block;
  for (long t = 0; t < frame.m_nCollisionBlocks; t++)
  {
    if (0 == seqCollTypes[t])
      frame.m_collision[t] = new tCollisionType0();
    else
    if (1 == seqCollTypes[t])
      frame.m_collision[t] = new tCollisionType1();
    else
    if (2 == seqCollTypes[t])
      frame.m_collision[t] = new tCollisionType2();
    else
    if (7 == seqCollTypes[t])
      frame.m_collision[t] = new tCollisionType7();
    else
    if (0x11 == seqCollTypes[t])
      frame.m_collision[t] = new tCollisionType11();
    else
    if (0x1d8 == seqCollTypes[t])
      frame.m_collision[t] = new tCollisionType1D8();
    else
      return REPORT_ERROR3(NULL, _T("Unknown collision type 0x%x for collision block %li at 0x%x"), seqCollTypes[t], t, 
                           pStream->getPosition() - 4*frame.m_nCollisionBlocks);
  }
  //
  // read respective collision blocks:
  long nCurrentScene = 0;
  long numBones = 0;
  for (long t = 0; t < frame.m_nCollisionBlocks; t++)
  {
    if (0x11 == seqCollTypes[t])
    {
      if (nCurrentScene >= frame.m_nHierarchySets)
        return REPORT_ERROR2(NULL, _T("Collision of type x11 #%li, while only %li hierarchy sets available."), nCurrentScene+1, frame.m_nHierarchySets);
      numBones = frame.m_hierarchy[nCurrentScene].m_numA[0];
      nCurrentScene++;
    }
    frame.m_collision[t]->readFromStream(pStream, numBones);
  }

  return ZRESULT_OK;
}


};//io

};//core
