/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  MafiaII import frame resource file
 *--------------------------------------------------------------------------
 * @histroy: 19.09.2010. Created
 * @histroy: 07.12.2012. Remastering for ZModeler3
 */
#include "MafiaII.h" 
#include "float16.h"
#include <helpers/serializeable.h>
#include <zmsharedclasses.h>


namespace core
{

namespace io
{

//
// helper routine to read qword hash; word length and ZString;
void readHashLenString(tHashNamePack* ptr, core::io::IStream* pStream)
{
  pStream->read(&ptr->m_dwHash, 8);
  pStream->read(&ptr->m_wNameLen, 2);
  ptr->m_strName.length(ptr->m_wNameLen);
  pStream->read((LPTSTR)ptr->m_strName, ptr->m_wNameLen);
}


//-------------------------------------------------------
// @name : CMafiaII::importFrameResource
// The import routine for frame resource
// @param tFrameResource frame  : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMafiaII::importFrameResource(tFrameResource& frame, core::io::IStream* pStream, long nID)
{
  if (!pStream)
    return ZRESULT_FALSE;

  //
  // grouping information:
  //
  pStream->read(&frame.m_bGrouping, 1 + 7*4);
  if (frame.m_bGrouping != 0)
  {
    //
    // root grouping information is available; probably scenery resource;
    pStream->read(&frame.m_grouping, 16+2+12*4+1);
    // read grouping names:
    frame.m_grouping.m_rootGroops.length(frame.m_nRootGroups);
    for (long t = 0; t < frame.m_nRootGroups; t++)
    {
      readHashLenString(&frame.m_grouping.m_rootGroops[t], pStream);
      // dump;
    }
  }
  //
  // model definitions:
  // 
  frame.m_models.length(frame.m_nGeomDefinitions);
  for (long t = 0; t < frame.m_nGeomDefinitions; t++)
    if (ZRESULT_OK != readGeomDefinition(frame.m_models[t], pStream))
      return ZRESULT_FAIL;

  //
  // end of lods
  DEBUG_MESSAGE1(_T("\n\n## end of import at 0x%x\n"), pStream->getPosition());

  //
  // unknown model partition? or collision
  //
  frame.m_modelUnks.length(frame.m_nModelUnkDefs);
  for (long t = 0; t < frame.m_nModelUnkDefs; t++)
    if (ZRESULT_OK != readModelsMaterialAssignment(frame.m_modelUnks[t], pStream))
      return ZRESULT_FAIL;

  //
  // end of lods
  DEBUG_MESSAGE1(_T("\n\n## end of material assignment chunks 0x%x\n"), pStream->getPosition());

  //
  // read scene composition set;
  frame.m_sceneSets.length(frame.m_nSceneCompositions);
  for (long t = 0; t < frame.m_nSceneCompositions; t++)
    if (ZRESULT_OK != readSceneCompositionSet(frame.m_sceneSets[t], pStream))
      return ZRESULT_FAIL;

  DEBUG_MESSAGE1(_T("\n\n## end of scene composition chunks 0x%x\n"), pStream->getPosition());

  //
  // read hierarchy compositions set;
  frame.m_hierarchy.length(frame.m_nHierarchySets);
  for (long t = 0; t < frame.m_nHierarchySets; t++)
    if (ZRESULT_OK != readHierarchyAssembly(frame.m_hierarchy[t], pStream))
      return ZRESULT_FAIL;

  //
  // read parenting info:
  frame.m_setParenting.length(frame.m_nHierarchyParenting);
  for (long t = 0; t < frame.m_nHierarchyParenting; t++)
    if (ZRESULT_OK != readParentingInfo(frame.m_setParenting[t], pStream))
      return ZRESULT_FAIL;

  
  DEBUG_MESSAGE1(_T("\n\n## end of hierarchy chunks 0x%x\n"), pStream->getPosition());

  if (ZFAILED(readCollisionBlocks(frame, pStream)))
    return ZRESULT_FAIL;

  DEBUG_MESSAGE1(_T("\n\n## end of collision chunks 0x%x\n"), pStream->getPosition());

  return ZRESULT_OK;
}



//-------------------------------------------------------
// @name : CMafiaII::readSpacePartition
// Description 
// @param tSpacePartition& partition     : 
// @param core::io::IStream* pStream     : 
//
// @return ZRESULT  : 
//-------------------------------------------------------
ZRESULT CMafiaII::readSpacePartition(tSpacePartition& p, core::io::IStream* pStream)
{
  pStream->read(&p.m_dwMemRequireA, 4*3+1);
  if (p.m_dwMemRequireA != p.m_dwMemRequireB ||
      p.m_nPartitionType != 3 && p.m_nPartitionType != 4)
      return REPORT_ERROR3( NULL, _T("Invalid Partition info: 0x%x/0x%x, type: %li"), 
                            p.m_dwMemRequireA, p.m_dwMemRequireB, p.m_nPartitionType);

  if (p.m_bAvailA)
  {
    // inplace array follows:
    pStream->read(p.m_fUnk3, 6*4 + 4);

    // dump:

    p.m_seqDesc1.length(p.m_numDesc1);
    pStream->read(p.m_seqDesc1.getBuffer(), p.m_numDesc1*sizeof(tDescriptor));

    // dump:
  }
  else
    p.m_numDesc1 = 0;


  //-- partition set #1 block
  pStream->read(&p.m_numLongs1, 4+1);
  if (p.m_numLongs1 != p.m_numDesc1 + 1)
    return REPORT_ERROR2(NULL, _T("Partition set #1 amount: %li, descs: %li"), p.m_numLongs1, p.m_numDesc1)
  if (p.m_bAvailB)
  {
    p.m_seqLongs1.length(p.m_numLongs1);
    pStream->read(p.m_seqLongs1.getBuffer(), 4*p.m_numLongs1);

    // dump
  }

  //-- faces sorting set #2 block
  pStream->read(&p.m_numLongs2, 4+1);
  if (p.m_bAvailC)
  {
    p.m_seqLongs2.length(p.m_numLongs2);
    pStream->read(p.m_seqLongs2.getBuffer(), 4*p.m_numLongs2);

    // dump
  }

  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CMafiaII::readMaterialSplit
// Material split information; 
// @param tMaterialSpltInfo& split       : 
// @param core::io::IStream* pStream     : 
//
// @return ZRESULT  : 
//-------------------------------------------------------
ZRESULT CMafiaII::readMaterialSplit(tMaterialSpltInfo& split, core::io::IStream* pStream, long nType)
{
  memset(&split, 0, 13*4+1);
  if (0xC == nType)
  {
    pStream->read(&split.m_nUnk18, 4+4+4+4+4+4);
    if (split.m_nUnk18 + split.m_nUnk20 + split.m_nUnk21 + 
        split.m_nSizeOfMatBurstEntries + split.m_nSizeOfMatSplitEntries != 0xE)
      return REPORT_ERROR1(NULL, _T("Invalid material split at 0x%x"), pStream->getPosition() - 24);
  }
  else
  if (1 == nType)
    split.m_numSplitGroup = nType;
  else
    return REPORT_ERROR(NULL, _T("Unexpected split type given"));

  if (1 == split.m_numSplitGroup)
  {
    pStream->read(&split.m_bAvailD, 1+4+4+4+4+4+8);
    if (split.m_nUnk24 + split.m_nSizeOfMatBurstEntries + split.m_nSizeOfMatSplitEntries != 0x14 + 0xC + 1)
      return REPORT_ERROR1(NULL, _T("Invalid material split info at 0x%x"), pStream->getPosition() - 1-20-8);

    // dump:
    DEBUG_MESSAGE4(_T("\n#material split info. bursts: %li, splits: %li\n Verts: %li, Faces: %li\n"), 
                  split.m_numMatBurst, split.m_numMatSplit,
                  split.m_numVerts, split.m_numFaces);

    // read burst info:
    split.m_seqMatBurst.length(split.m_numMatBurst);
    pStream->read(split.m_seqMatBurst.getBuffer(), split.m_numMatBurst*sizeof(tMaterialBurst));
    // dump:
    for (long t = 0; t < split.m_numMatBurst; t++)
    {
      DEBUG_MESSAGE5("  burst %li:\t%hi .. %hi\ti1:%hi\ti2:%hi\n", t, 
                    split.m_seqMatBurst[t].m_firstIndex, split.m_seqMatBurst[t].m_lastIndex, 
                    split.m_seqMatBurst[t].m_nNeighbourIndex1, split.m_seqMatBurst[t].m_nNeighbourIndex2);
    }

    // read split info:
    split.m_seqMatSplit.length(split.m_numMatSplit);
    pStream->read(split.m_seqMatSplit.getBuffer(), split.m_numMatSplit*sizeof(tMaterialSplit));

    // dump:
    for (long t = 0; t < split.m_numMatSplit; t++)
    {
      DEBUG_MESSAGE4("  split %li: (bursts %li from %li), base index:%li\n", t, 
                    split.m_seqMatSplit[t].m_numBurst, split.m_seqMatSplit[t].m_nFirstBurst, split.m_seqMatSplit[t].m_nBaseIndex);
    }
  }


  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CMafiaII::readGeomDefinition
// Description 
// @param tGeometryModelDefinition& model                                     : 
// @param core::io::IStream* pStream     : 
//
// @return ZRESULT  : 
//-------------------------------------------------------
ZRESULT CMafiaII::readGeomDefinition(tGeometryModelDefinition& model, core::io::IStream* pStream)
{
  pStream->read(&model.m_nLODs, 3+4*4);
  //if (0 != model.m_nZero)
  //  return REPORT_ERROR1(NULL, _T("Model LOD info is wrong at 0x%x"), pStream->getPosition()-3-4*4);

  //
  // dump bounds
  //
  DEBUG_MESSAGE6("\nGeomLod. Lods: %li, Zero: %li\tBounds: %f, %f, %f, %f\n",
                (long)model.m_nLODs, (long)model.m_nZero, 
                model.m_fBounds[0], model.m_fBounds[1], model.m_fBounds[2], model.m_fBounds[3]);

  model.m_LODs.length(model.m_nLODs);
  for (long nLod = 0; nLod < model.m_nLODs; nLod++)
  {
    //
    // read the lod;
    tGeometryLOD& lod = model.m_LODs[nLod];

    pStream->read(&lod.m_fUnk2, 4);
    readHashLenString(&lod.m_nameIB, pStream);
    pStream->read(&lod.m_dwVertsInfo, 4);
    readHashLenString(&lod.m_nameVB, pStream);

    pStream->read(&lod.m_numVertsInBuf, 4 + 4 + 4);

    //
    // read partition:
    if (1 == lod.m_nPartition)
    {
      if (ZRESULT_OK != readSpacePartition(lod.partition, pStream))
        return ZRESULT_FAIL;
    }
    else
    if (0 != lod.m_nPartition)
      return REPORT_ERROR1(NULL, _T("Invalid partition type specified at 0x%x"), pStream->getPosition()-4);

    pStream->read(&lod.m_nMatSplitType, 4);
    if (0xC == lod.m_nMatSplitType || 0x1 == lod.m_nMatSplitType)
    {
      if (ZRESULT_OK != readMaterialSplit(lod.m_matSplit, pStream, lod.m_nMatSplitType))
        return ZRESULT_FAIL;
    }
    else
    if (0 != lod.m_nMatSplitType)
      return REPORT_ERROR1(NULL, _T("Invalid material split type specified at 0x%x"), pStream->getPosition()-4);

    pStream->read(&lod.m_nZeroTail, 4);
    if (0 != lod.m_nZeroTail)
      return REPORT_ERROR1(NULL, _T("Non-zero LOD information tail at 0x%x"), pStream->getPosition()-4);
  }

  return ZRESULT_OK;
}



//-------------------------------------------------------
// @name : CMafiaII::readModelsMaterialAssignment
// Reads unknown definition or space partition. 
// @param tLODDescriptor& desc           : 
// @param core::io::IStream* pStream     : 
//
// @return ZRESULT  : 
//-------------------------------------------------------
ZRESULT CMafiaII::readModelsMaterialAssignment(tLODDescriptor& desc, core::io::IStream* pStream)
{
  pStream->read(&desc.m_nLods, 1);
  desc.m_assign.length(desc.m_nLods);
  long num = 0;
  for (long t = 0; t < desc.m_nLods; t++)
  {
    pStream->read(&num, 4);
    if (num > 0xFFFF)
      return REPORT_ERROR1(NULL, _T("Unexpected value in material assignment chunk at 0x%x"), pStream->getPosition());
    desc.m_assign[t].length(num);
  }
  // read bounds:
  pStream->read(desc.m_fBounds, 4*6);
  // dump:
  DEBUG_MESSAGE7("Material assginemnts for %li lods; {%f, %f, %f} x {%f, %f, %f}\n", (long)desc.m_nLods, 
          desc.m_fBounds[0], desc.m_fBounds[1], desc.m_fBounds[2], 
          desc.m_fBounds[3], desc.m_fBounds[4], desc.m_fBounds[5]);

  // read actual descriptors:
  for (long t = 0; t < desc.m_nLods; t++)
  {
    pStream->read(desc.m_assign[t].getBuffer(), desc.m_assign[t].length()*sizeof(tMaterialAssignFragment));

    // dump :
    DEBUG_MESSAGE2(" Frame Desc for LOD %li of %li\n", t+1, (long)(desc.m_nLods));
    for (long d = 0; d < desc.m_assign[t].length(); d++)
    {
      tMaterialAssignFragment& ma = desc.m_assign[t][d];
      DEBUG_MESSAGE5("  Mat assginment %li: %li tris from index # %li, using 0x%.8x%.8x\n", d, ma.m_numTris, ma.m_nFirstIndex,
        (DWORD)((ma.m_matHash&0xFFFFFFFF00000000i64)>>32), (DWORD)(ma.m_matHash&0xFFFFFFFFi64));
    }
  }
  return ZRESULT_OK;
}



//-------------------------------------------------------
// @name : CMafiaII::readSceneCompositionSet
// Description 
// @param tSceneCompositionSet& sceneSet : 
// @param core::io::IStream* pStream     : 
//
// @return ZRESULT  : 
//-------------------------------------------------------
ZRESULT CMafiaII::readSceneCompositionSet(tSceneCompositionSet& sceneSet, core::io::IStream* pStream)
{
  pStream->read(&sceneSet.m_nBlocks, 4);    //amount of bones?
  pStream->read(&sceneSet.m_nSet, 1);       //amount of sets;
  sceneSet.m_seq.length(sceneSet.m_nSet);
  sceneSet.m_blocks.length(sceneSet.m_nBlocks);
  //
  DEBUG_MESSAGE2(" Composition blocks: %li, sets: %li\n", sceneSet.m_nBlocks, (long)sceneSet.m_nSet);

  for (long t = 0; t < sceneSet.m_nSet; t++)
  {
    pStream->read(&sceneSet.m_seq[t].m_nIDs, 8);
    DEBUG_MESSAGE2("  Set IDs: %li, %li\n", sceneSet.m_seq[t].m_nIDs, sceneSet.m_seq[t].m_nNums);
  }
  // bounds?
  pStream->read(&sceneSet.m_bounds, sizeof(tBound));
  // blocks:
  pStream->read(sceneSet.m_blocks.getBuffer(), sceneSet.m_nBlocks*sizeof(tSceneCompose1));

  // dump blocks:
  for (long t = 0; t < sceneSet.m_nBlocks; t++)
  {
    tSceneCompose1& c = sceneSet.m_blocks[t];
    float f1 = SQRTF(c.m_mat4x3[0].x*c.m_mat4x3[0].x + c.m_mat4x3[0].y*c.m_mat4x3[0].y + c.m_mat4x3[0].z*c.m_mat4x3[0].z);
    float f2 = SQRTF(c.m_mat4x3[1].x*c.m_mat4x3[1].x + c.m_mat4x3[1].y*c.m_mat4x3[1].y + c.m_mat4x3[1].z*c.m_mat4x3[1].z);
    float f3 = SQRTF(c.m_mat4x3[2].x*c.m_mat4x3[2].x + c.m_mat4x3[2].y*c.m_mat4x3[2].y + c.m_mat4x3[2].z*c.m_mat4x3[2].z);
    DEBUG_MESSAGE2("   Block %3li: %3li\t", t, c.m_nUnk == 0xFF ? -1 : (long)c.m_nUnk);
    DEBUG_MESSAGE6(" {%.3f %.3f %.3f} at {%.3f %.3f %.3f}\n", f1,f2,f3,c.m_mat4x3[0].w, c.m_mat4x3[1].w, c.m_mat4x3[2].w);
  }

  // read seq:
  for (long t = 0; t < sceneSet.m_nSet; t++)
  {
    tSceneCompositionInfo& info = sceneSet.m_seq[t];
    pStream->read(info.m_bytes, 8);
    info.m_IDs.length(sceneSet.m_seq[t].m_nIDs);
    info.m_nums.length(sceneSet.m_seq[t].m_nNums);
    pStream->read(info.m_IDs.getBuffer(), sceneSet.m_seq[t].m_nIDs);
    pStream->read(info.m_nums.getBuffer(), 2*sceneSet.m_seq[t].m_nNums);

    //
    // dump:
    DEBUG_MESSAGE8("  set %li of %li: IDs: %li, nums: %li, [%li, %li, %li, %li]\n", 
                  t+1, (long)sceneSet.m_nSet, info.m_nIDs, info.m_nNums, 
                  (long)info.m_bytes[0], (long)info.m_bytes[1], (long)info.m_bytes[2], (long)info.m_bytes[3]);
    DEBUG_MESSAGE1(" ID bytes (%li):", info.m_IDs.length());
    for (long g = 0; g < info.m_IDs.length(); g++)
    {
      DEBUG_MESSAGE2("%li%c", (long)info.m_IDs[g], g == info.m_IDs.length()-1 ? '\n' : ',');
    }
    DEBUG_MESSAGE1(" Flag WORDs (%li):", info.m_nums.length());
    for (long g = 0; g < info.m_nums.length(); g++)
    {
      DEBUG_MESSAGE2("%hx%c", info.m_nums[g], g == info.m_nums.length()-1 ? '\n' : ',');
    }
  }
  return ZRESULT_OK;
}

//
// helper dump routine;
// 
inline void dumpBytesArrayInfo(LPCTSTR pszLeadingStr, tBYTESeq& seq, LPCTSTR pszTailStr)
{
  BYTE nMin = 255;
  BYTE nMax = 0;
  for (long t = 0; t < seq.length(); t++)
  {
    if (nMin > seq[t])
      nMin = seq[t];
    if (nMax < seq[t])
      nMax = seq[t];
  }
  DEBUG_MESSAGE7("%s Num: %li; Min: %li; Max: %li; First: %li; Last: %li;%s", pszLeadingStr,
                 seq.length(), (long)nMin, (long)nMax, (long)seq[0], (long)seq[seq.length()-1], 
                 pszTailStr);
}


//-------------------------------------------------------
// @name : CMafiaII::readHierarchyAssembly
// Reads hierarchy assembly. poorly decoded; 
// @param tHierarchyAssembly& hie        : 
// @param core::io::IStream* pStream     : 
//
// @return ZRESULT  : 
//-------------------------------------------------------
ZRESULT CMafiaII::readHierarchyAssembly(tHierarchyAssembly& hie, core::io::IStream* pStream)
{
  pStream->read(hie.m_numA, 16+4+4);
  hie.m_numInUnkSet.length(hie.m_numUnkSet);
  hie.m_setUnk.length(hie.m_numUnkSet); //will be read later;
  pStream->read(hie.m_numInUnkSet.getBuffer(), 4*hie.m_numUnkSet);
  pStream->read(&hie.m_idType, 1);
  // dump
  DEBUG_MESSAGE6("\nHierarchy assembly: %li %li %li %li, %li, sets %li: ",
                  hie.m_numA[0], hie.m_numA[1], hie.m_numA[2], hie.m_numA[3], hie.m_numB, hie.m_numUnkSet);
  for (long t = 0; t < hie.m_numUnkSet; t++)
  {
    //dump:
    DEBUG_MESSAGE2("%li%c", hie.m_numInUnkSet[t], t == hie.m_numUnkSet-1 ? '\n' : ' ');
  }
  DEBUG_MESSAGE1(" Names subtype: %li\n", (long)hie.m_idType);
  // read nodes identifications:
  hie.m_nodes.length(hie.m_numA[0]);
  for (long t = 0; t < hie.m_numA[0]; t++)
    readHashLenString(&hie.m_nodes[t], pStream);
    // will dump later;

  // read matrices:
  hie.m_nodeMats.length(hie.m_numA[1]);
  hie.m_nodeMats2.length(hie.m_numA[1]);
  pStream->read(hie.m_nodeMats.getBuffer(), 0x30*hie.m_numA[1]);
  pStream->read(&hie.m_numA2, 4);
  hie.m_jointTypes.length(hie.m_numA2);
  pStream->read(hie.m_jointTypes.getBuffer(), hie.m_numA2); //will dump later;
  pStream->read(hie.m_nodeMats2.getBuffer(), 0x30*hie.m_numA[1]);

  //
  // dump block with names, types, offsets;
  for (long t = 0; t < hie.m_numA[0]; t++)
  {
    // unified scale on matrix1 :
    ZVector vScale1 = ZVector(SQRTF(hie.m_nodeMats[t][0]^hie.m_nodeMats[t][0]),
                              SQRTF(hie.m_nodeMats[t][1]^hie.m_nodeMats[t][1]),
                              SQRTF(hie.m_nodeMats[t][2]^hie.m_nodeMats[t][2]));
    ZVector vScale2 = ZVector(SQRTF(hie.m_nodeMats2[t][0]^hie.m_nodeMats2[t][0]),
                              SQRTF(hie.m_nodeMats2[t][1]^hie.m_nodeMats2[t][1]),
                              SQRTF(hie.m_nodeMats2[t][2]^hie.m_nodeMats2[t][2]));
                
    DEBUG_MESSAGE5(_T("Name :%3li: %24s; type: %2li\tscale: {%.3f, %.3f} \n"), t, (LPCTSTR)hie.m_nodes[t].m_strName, 
                  (long)hie.m_jointTypes[t], 0.33333f*(vScale1^vScale1), 0.33333f*(vScale2^vScale2));
  }

  // read float+(UNKNOWN YET) bytes section:
  for (long d = 0; d < hie.m_numUnkSet; d++)
  {
    tUnknownHierarchySet& s = hie.m_setUnk[d];
    s.m_seqBounds.length(hie.m_numA[2]);
    s.m_seqBytes1.length(hie.m_numA[2]);  // considerable
    s.m_seqBytes2.length(hie.m_numInUnkSet[d]);  // considerable
    pStream->read(s.m_seqBounds.getBuffer(), hie.m_numA[2]*sizeof(tBound));
    pStream->read(&s.m_nBytes, 1);//should be 0?
    pStream->read(s.m_seqBytes1.getBuffer(), s.m_seqBytes1.length());
    pStream->read(s.m_seqBytes2.getBuffer(), s.m_seqBytes2.length());
    
    // dump?
    DEBUG_MESSAGE3(" Unk Secion (%li of %li), %li\n", d+1, hie.m_numUnkSet, (long)s.m_nBytes);
    dumpBytesArrayInfo("  Set#1: ", s.m_seqBytes1, "\n    ");
    for (long t = 0; t < s.m_seqBytes1.length(); t++)
    {
      //dump:
      DEBUG_MESSAGE2("%li%c", (long)s.m_seqBytes1[t], t == s.m_seqBytes1.length()-1 ? '\n' : ' ');
    }
    dumpBytesArrayInfo("  Set#2: ", s.m_seqBytes2, "\n    ");
    for (long t = 0; t < s.m_seqBytes2.length(); t++)
    {
      //dump:
      DEBUG_MESSAGE2("%li%c", (long)s.m_seqBytes2[t], t == s.m_seqBytes2.length()-1 ? '\n' : ' ');
    }
  }

  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CMafiaII::readParentingInfo
// reads hierarchy parenting bytes information. poorly decoded; 
// @param tHierarchyParenting& par       : 
// @param core::io::IStream* pStream     : 
//
// @return ZRESULT  : 
//-------------------------------------------------------
ZRESULT CMafiaII::readParentingInfo(tHierarchyParenting& par, core::io::IStream* pStream)
{
  // read parenting info:
  pStream->read(&par.m_num, 4);
  par.m_seqParents.length(par.m_num);
  pStream->read(par.m_seqParents.getBuffer(), par.m_num);
  //
  // considerable;
  pStream->read(&par.n1, 1);
  pStream->read(&par.n2, 1);
  par.m_set1.length(par.m_num);
  par.m_set2.length(par.m_num);
  pStream->read(par.m_set1.getBuffer(), par.m_num);
  pStream->read(par.m_set2.getBuffer(), par.m_num);
  //
  // dump;
  DEBUG_MESSAGE3(" Parenting info for %li items (%li, %li)\n", par.m_num, (long)par.n1, (long)par.n2);
  dumpBytesArrayInfo("  block #1: ", par.m_set1, "\n    ");
  for (long t = 0; t < par.m_set1.length(); t++)
  {
    //dump:
    DEBUG_MESSAGE2("%li%c", (long)par.m_set1[t], t == par.m_set1.length()-1 ? '\n' : ' ');
  }
  dumpBytesArrayInfo("  block #2: ", par.m_set2, "\n    ");
  for (long t = 0; t < par.m_set2.length(); t++)
  {
    //dump:
    DEBUG_MESSAGE2("%li%c", (long)par.m_set2[t], t == par.m_set2.length()-1 ? '\n' : ' ');
  }

  return ZRESULT_OK;
}

};//io

};//core
