/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  MafiaII index buffer pool import routine
 *--------------------------------------------------------------------------
 * @histroy: 08.09.2010. Created
 * @histroy: 07.12.2012. Remastering for ZModeler3
 */
#include "MafiaII.h" 
#include <helpers/serializeable.h>
#include <zmsharedclasses.h>


namespace core
{

namespace io
{

//-------------------------------------------------------
// @name : CMafiaII::importCurrentStream
// The import routine
// @param core::IProcParams* pParams     : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMafiaII::importIndexBufferPool(tIndicesPool& pool, core::io::IStream* pStream, long nID)
{
  if (!pStream)
    return ZRESULT_FALSE;

  //load indices buffer pool;
  BYTE bID = 0;
  pStream->read(&bID, 1);
  if (bID != MAFIAII_INDEX_POOL_ID)
    return REPORT_ERROR2(NULL, "IndexBufferPool_%li ID mismatch %li.", nID, (long)bID);
  pStream->read(&pool, 8);
  DEBUG_MESSAGE3("Indices pool #%li: %li streams, 0x%x VRAM\n", nID, pool.m_nStreams, pool.m_nVRAM);
  pool.m_Buffers.length(pool.m_nStreams);
  for (long nStream =  0; nStream < pool.m_nStreams; nStream++)
  {
    tIndicesBuffer& buf = pool.m_Buffers[nStream];
    pStream->read(&buf, 16);
    DEBUG_MESSAGE3("  Buffer %li: {%li, size: 0x%x}\n", nStream, buf.m_nType, buf.m_nSize);
    buf.m_indices.length(buf.m_nSize>>1);
    pStream->read(buf.m_indices.getBuffer(), buf.m_nSize);
  }
  return ZRESULT_OK;
}


};//io

};//core
