/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  MafiaII import routines
 *--------------------------------------------------------------------------
 * @histroy: 07.09.2010. Created
 * @histroy: 07.12.2012. Remastering for ZModeler3
 */
#include "MafiaII.h" 
#include <helpers/serializeable.h>
#include <helpers/nodes.h>
#include <helpers/sceneNavigation.h>
#include <zmsharedclasses.h>
#include <core/ui/controls/IUserDefinedOptions.h>
#include "float16.h"


DEBUG_FILE()

namespace core
{

namespace io
{


//-------------------------------------------------------
// @name : CMafiaII::importCurrentStream
// The import routine
// @param core::IProcParams* pParams     : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMafiaII::importCurrentStream(core::IProcParams* pParams, DWORD nMode, ZBlackBox* pBox)
{
  // prepare services:
  services::getTexturesService(&m_pTexSrv);
  services::getMaterialsService(&m_pMatSrv);
  // obtain scene and root node;
  g_pZModeler->getScene(&m_pScene);
  //m_pScene->getRoot(&m_pFileRoot);
  if (!m_pScene)
    return REPORT_ERROR(NULL, _T("No scene."));
  m_pScene->getNodeService(&m_pNodeSrv);
  m_pParams = pParams;

  DEBUG_OPEN()

  tIndicesPool      IndicesPool;
  tVerticesPool     VerticesPool;
  tFrameResource    frame;
  tFrameNameTable   frameNameTable;

  //
  // load required resources and pools:
  char szFileName[128];

  ZPtr<core::io::IStream> pStream;
  if (ZRESULT_OK != createSharedClass(_T(ZM_CLASS_FILESTREAM), IID_(core::io::IStream), (void**)&pStream))
    return ZRESULT_FALSE;
  //
  // frame names table:
  //
  _stprintf(szFileName, "FrameNameTable_%li.bin", 0);
  if (ZRESULT_OK == pStream->open(szFileName))
  {
    importFrameNamesTable(frameNameTable, pStream, 0);
    pStream->close();
  }

  //
  // the frame resource:
  //
  _stprintf(szFileName, "FrameResource_%li.bin", 0);
  if (ZRESULT_OK == pStream->open(szFileName))
  {
    importFrameResource(frame, pStream, 0);
    pStream->close();
  }

  _stprintf(szFileName, "VertexBufferPool_%li.bin", 0);
  if (ZRESULT_OK == pStream->open(szFileName))
  {
    importVertexBufferPool(VerticesPool, pStream, 0);
    pStream->close();
  }
  _stprintf(szFileName, "IndexBufferPool_%li.bin", 0);
  if (ZRESULT_OK == pStream->open(szFileName))
  {
    importIndexBufferPool(IndicesPool, pStream, 0);
    pStream->close();
  }
  
  //
  // create the scene;
  return createScene(m_pScene, frameNameTable, frame, IndicesPool, VerticesPool);
}


};//io

};//core
