/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  MafiaII index buffer pool import routine
 *--------------------------------------------------------------------------
 * @histroy: 08.09.2010. Created
 * @histroy: 07.12.2012. Remastering for ZModeler3
 */
#include "MafiaII.h" 
#include <helpers/serializeable.h>
#include <zmsharedclasses.h>


namespace core
{

namespace io
{

//-------------------------------------------------------
// @name : CMafiaII::importVertexBufferPool
// Comment  
// @param tVerticesPool& pool            : 
// @param long nID                       : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMafiaII::importVertexBufferPool(tVerticesPool& pool, core::io::IStream* pStream, long nID)
{
  //
  // create stream on indices buffer file
  //char szFileName[128];
  //_stprintf(szFileName, "VertexBufferPool_%li.bin", nID);

  //ZPtr<core::io::IStream> pStream;
  //if (ZRESULT_OK == createSharedClass(_T("core::io::CFileStream"), 
  //                  IID_(core::io::IStream), (void**)&pStream) &&
  //    ZRESULT_OK == pStream->open(szFileName, false, false, true))
  if (!pStream)
    return ZRESULT_FALSE;

  //load indices buffer pool;
  BYTE bID = 0;
  pStream->read(&bID, 1);
  if (bID != MAFIAII_VERTEX_POOL_ID)
    return REPORT_ERROR2(NULL, "VertexBufferPool_%li ID mismatch %li.", nID, (long)bID);
  pStream->read(&pool, 8);
  DEBUG_MESSAGE3("\nVertices pool #%li: %li streams, 0x%x VRAM\n", nID, pool.m_nStreams, pool.m_nVRAM);
  pool.m_Buffers.length(pool.m_nStreams);
  for (long nStream =  0; nStream < pool.m_nStreams; nStream++)
  {
    tVerticesBuffer& buf = pool.m_Buffers[nStream];
    pStream->read(&buf, 12);
    DEBUG_MESSAGE2("  Buffer %li: size: 0x%x}\n", nStream, buf.m_nSize);
    buf.m_vertices.length(buf.m_nSize);
    pStream->read(buf.m_vertices.getBuffer(), buf.m_nSize);
    /*
    //
    // make a quick test:
    if (0 == nStream)
    {
      struct tBigVert
      {
        WORD  x,y,z; //something is wrong with Z?
        char  n[6];
        //deformation behaviour:
        DWORD c;     //bytes weight: w0, w1, w2, w3
        DWORD d;     //bytes boneid: i0, i1, i2, i3
        //mesh separation:
        DWORD e;     //mask?       :  0   0  FF  FF
        DWORD f;     //property IDs:  0x17 - default;
        WORD  uv0[2];
        WORD  uv1[2];
        WORD  uv2[2];
      };
      tBigVert* pVert = reinterpret_cast<tBigVert*>(buf.m_vertices.getBuffer());
      for (long n = 0; n < buf.m_nSize/(signed)sizeof(tBigVert); n++)
      {
        //if (pVert[n].z >= 0x8000)
        //  pVert[n].z -= 0x8000;
        //memset(pVert[n].n, 0x80, 6);
        //DEBUG_MESSAGE5("Ver: %6li: 0x%8x 0x%8x 0x%8x 0x%8x\n", n, pVert[n].c, pVert[n].d, pVert[n].e, pVert[n].f);
        //pVert[n].f = 0x17171717;
        //pVert[n].e = 0xFFFF0000;
        //pVert[n].c = 0xFF;
        //pVert[n].d &= 0xFF;
        //pVert[n].n[0] = pVert[n].n[1] = pVert[n].n[5] = 0;

        //pVert[n].uv1[0] = 0;
        //pVert[n].uv1[1] = 0;
        //pVert[n].uv2[0] = 0;
        //pVert[n].uv2[1] = 0;
      }
      pStream->seekOffset(-buf.m_nSize);
      pStream->write(buf.m_vertices.getBuffer(), buf.m_nSize);
    }
    */
  }
  return ZRESULT_OK;
}


};//io

};//core
