/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  Mafia II main file
 *--------------------------------------------------------------------------
 * @histroy: 07.09.2010. Created
 * @histroy: 07.12.2012. Remastering for ZModeler3
 */
#include "MafiaII.h" 
#include <helpers/serializeable.h>
#include <helpers/nodes.h>
#include <helpers/undo.h>
#include <shared/ICrypt.h>
#include <zmsharedclasses.h>

namespace core
{

namespace io
{

INTERFACE_BEGINMAP(CMafiaII)
  INTERFACE_ENTRY(core::INamed)
  INTERFACE_ENTRY(core::io::IImportExportFilter)
  INTERFACE_ENTRY(core::IErrorInfo)
INTERFACE_ENDMAP()



CMafiaII::CMafiaII() : core::CErrorInfo()
{
  m_strTitle = _T("Mafia II .mafia");
  CMafiaIIDialog::CreateInstance(&m_pDlg);
  m_pDlg->m_pOwner  = this;

  // load profile
  ZPtr<services::IProfilingService> pProfiler;
  services::getProfilingService(&pProfiler);
  if (pProfiler)
  {
    ZStringSeq seqBranch;
    seqBranch.add(_T("Preference"));
    seqBranch.add(_T("Filters"));
    seqBranch.add(_T("MafiaII"));
  }
}

CMafiaII::~CMafiaII()
{
}

//-----------------------
// core::INamed
//-----------------------
ZRESULT CMafiaII::getName(ZString& str)
{
  str = _T("core::io::CMafiaII");
  return ZRESULT_OK;
}

ZRESULT CMafiaII::setName(ZString str)
{
  return ZRESULT_FALSE;
}

ZRESULT CMafiaII::getTitle(ZString& str)
{
  str = m_strTitle;
  return ZRESULT_OK;
}

ZRESULT CMafiaII::setTitle(ZString str)
{
  m_strTitle = str;
  return ZRESULT_OK;
}

ZRESULT CMafiaII::getDescription(ZStringSeq& seqDesc)
{
  return ZRESULT_FALSE;
}

ZRESULT CMafiaII::setDescription(ZStringSeq& seqDesc)
{
  return ZRESULT_FALSE;
}

//-----------------------
// IImportExportFilter
//-----------------------
ZRESULT CMafiaII::getExtensions(ZStringSeq& seqExt)
{
  seqExt.length(0);
  seqExt.add(_T("mafia"));
  return ZRESULT_OK;
}

ZRESULT CMafiaII::getExtensionSupport(LPCTSTR pszExt, DWORD& dwSupport)
{
  if (!_tcscmp(pszExt, _T("mafia")))
    dwSupport = ZM_FILTER_SET_SUPPORT_MASK(
          ZM_FILTER_CAN_IMPORT_BULK | ZM_FILTER_CAN_EXPORT_BULK,
          ZM_FILTER_SUPPORT_GEOMETRY);
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CMafiaII::setCurrentStream
// Sets current stream;
// @param core::io::IStream* pStream     : 
// @param LPCTSTR pszFileName            : 
// @param LPCTSTR pszPath                : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMafiaII::setCurrentStream(core::io::IStream* pStream, LPCTSTR pszFileName, LPCTSTR pszPath)
{
  m_pStream = pStream;
  m_strFileName = pszFileName;
  if (m_strFileName.length() &&
      _tcschr(m_strFileName, _T('.')))
    *_tcschr(m_strFileName, _T('.')) = 0;
  m_pRoot = NULL;
  //reset: clear maps, sequences, etc.
  m_pTexSrv   = NULL;
  m_pMatSrv   = NULL;
  m_pNodeSrv  = NULL;
  m_pScene    = NULL;
  m_pParams   = NULL;

  //
  // detect local path
  if (pStream!=NULL && pszPath!=NULL)
  {
  }

  return ZRESULT_OK;
}

ZRESULT CMafiaII::getExportDialog(core::layout::IChildDialog** ppDlg, DWORD nMode)
{
  ZM_AUTO_HANDLE_SWITCH();
  if (NULL == ppDlg)
    return ZRESULT_INVALID_ARG;
  if (!m_pDlg)
    return ZRESULT_FALSE;
  m_pDlg->setDialogMode(true);
  (*ppDlg) = static_cast<core::layout::IChildDialog*>(m_pDlg);
  (*ppDlg)->AddRef();
  return ZRESULT_OK;
}

ZRESULT CMafiaII::getImportDialog(core::layout::IChildDialog** ppDlg, DWORD nMode)
{
  ZM_AUTO_HANDLE_SWITCH();
  if (NULL == ppDlg)
    return ZRESULT_INVALID_ARG;
  if (!m_pDlg)
    return ZRESULT_FALSE;
  m_pDlg->setDialogMode(false);
  (*ppDlg) = static_cast<core::layout::IChildDialog*>(m_pDlg);
  (*ppDlg)->AddRef();
  return ZRESULT_OK;
}



//-------------------------------------------------------
// @name : CMafiaII::readFileInfo
// This will provide file info
// @param bool& bCanImport               : 
// @param tBYTESeq& seqThumbnailData     : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMafiaII::readFileInfo(bool& bCanImport, tBYTESeq& seqData)
{
  bCanImport = true;
  m_strComment.length(0);
  m_bLock = false;
  m_pDlg->setDialogMode(m_pDlg->getDialogMode());
  m_pDlg->setFileInfo(m_bLock, m_strComment);
  m_pStream->seekTo(0);
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CMafiaII::createMeshNode
// Helper routine: creates the mesh
// @param cene::IScene* pScene           : 
// @param scene::IMeshNode** ppMNode     : 
// @param scene::IPolyMesh** ppMesh      : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMafiaII::createMeshNode(
                            scene::IScene* pScene, 
                            scene::INode** ppNode,
                            scene::IMeshNode** ppMNode, 
                            scene::IPolyMesh** ppMesh)
{
  if (!m_pNodeSrv)
    return ZRESULT_FALSE;
  ZPtr<scene::IMeshNode>  pMNode;
  ZPtr<scene::IPolyMesh>  pPMesh;
  if (ZRESULT_OK == m_pNodeSrv->createNode(IID_(scene::IMeshNode), (void**)ppMNode))
  {
    if (ppNode)
      (*ppMNode)->QueryInterface(IID_(scene::INode), (void**)ppNode);
    createSharedClass(_T(ZM_CLASS_POLYMESH), IID_(scene::IPolyMesh), (void**)ppMesh);
    if (!*ppMesh)
      return REPORT_ERROR(NULL, _T("Failed to create mesh node"));
  }
  else
    return REPORT_ERROR(NULL, _T("Failed obtain meshnode from nodes service"));
  (*ppMNode)->setMesh(*ppMesh);
  return ZRESULT_OK;
}

//-------------------------------------------------------
// @name : CMafiaII::createDummyNode
// creates dummy nodes
// @param core::IProcParams* pParams     : 
// @param scene::IScene* pScene          : 
// @param LPCTSTR pszName                : 
// @param ZMatrix& matNode               : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMafiaII::createDummyNode(
                                    scene::INode*  pRoot, 
                                    LPCTSTR pszName,
                                    bool bSolid,
                                    long nShape, 
                                    DWORD dwColor,
                                    ZVector  vScale,
                                    ZMatrix& matNode,
                                    scene::IDummyNode** ppRes)
{
  //
  // create dummy helper node.
  //
  ZPtr<scene::IDummyNode>   pDummyNode;
  //create shared classes
  if (ZFAILED(m_pNodeSrv->createNode(IID_(scene::IDummyNode), (void**)&pDummyNode)))
    return ZRESULT_FAIL;
  //set node orientation
  ZPtr<scene::ITransformable> pTrans;
  if (ZRESULT_OK == pDummyNode->QueryInterface(IID_(scene::ITransformable), (void**)&pTrans))
    pTrans->setTransform(matNode);
  pDummyNode->setSize(vScale.x, vScale.y, vScale.z);
  pDummyNode->setDisplayInfo(dwColor, bSolid, nShape);
  //set name
  core::setINamedTitle(pDummyNode, pszName);

  //create common undo
  core::undo::undoNodeOperation(ZString("Create ") + (pszName ? ZString(pszName) : ZString(_T(" Object"))),
    core::undo::UNDO_NODE_ADD, m_pParams, pDummyNode);

  //attach to scene
  m_pScene->attachNode(pDummyNode, pRoot);
  if (ppRes)
    pDummyNode.CopyTo(ppRes);
  return ZRESULT_OK;
}


//-------------------------------------------------------
// @name : CMafiaII::attachSceneObject
// Attaches scene node (creates undo), sets object title
// @param scene::INode* pNode            : 
// @param scene::INode* pParent          : 
// @param LPCTSTR pszName                : 
//
// @return ZRESULT : 
//-------------------------------------------------------
ZRESULT CMafiaII::attachSceneObject(scene::INode* pNode, 
                                        scene::INode* pParent,
                                        LPCTSTR pszName,
                                        ZMatrix& mat)
{
  if (NULL == pNode)
    return ZRESULT_INVALID_ARG;
  if (NULL != pszName)
    core::setINamedTitle(pNode, pszName);
  scene::setITransformableMatrix(pNode, mat);
  //create common undo
  core::undo::undoNodeOperation(pszName ? (ZString("Create ") + ZString(pszName)):_T("Create node"), 
                                core::undo::UNDO_NODE_ADD, m_pParams, pNode);
  pNode->changed(NC_VISUAL | NC_GEOMETRY);
  //attach to scene
  m_pScene->attachNode(pNode, pParent);
  return ZRESULT_OK;
}



//-------------------------------------------------------
// @name : CMafiaII::locateSceneNode
// Searches for node with supplied name
// @param scene::INode* pNode            : 
// @param LPCTSTR pszName                : 
//
// @return scene::IDummyNode* : 
//-------------------------------------------------------
scene::IDummyNode* CMafiaII::locateSceneNode(scene::INode* pNode, LPCTSTR pszName)
{
  if (NULL == pNode || NULL == pszName)
    return NULL;
  ZPtr<scene::IDummyNode> pDummy;
  ZString strName;
  core::getINamedTitle(pNode, strName);
  if (strName.length() && !_tcsicmp(strName, pszName))
  {
    if (ZRESULT_OK == pNode->QueryInterface(IID_(scene::IDummyNode), (void**)&pDummy))
      return pDummy;
    return NULL; 
  }
  ZPtr<scene::INode>    ptr;
  ZPtr<scene::CNodeCol> pCol;
  pNode->getChildren(&pCol);
  if (!pCol)
    return NULL;
  pNode = NULL;
  pCol->pushState();
  pCol->reset();
  while (ZRESULT_OK == pCol->next(&ptr))
  {
    if (NULL != (pNode = locateSceneNode(ptr, pszName)))
      break;
    ptr = NULL;
  }
  pCol->popState();
  if (pNode && ZRESULT_OK == pNode->QueryInterface(IID_(scene::IDummyNode), (void**)&pDummy))
    return pDummy;
  return NULL; 
}


};//core

};//io

