/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 *--------------------------------------------------------------------------
 * @purpose:  MafiaII collision data (frame resource fragment) import;
 *--------------------------------------------------------------------------
 * @histroy: 28.12.2012. Created
 */
#include "MafiaII.h" 


namespace core
{

namespace io
{


//-------------------------------------------------------
// @name : CMafiaII::createScene
// Creates the scene;
// @param scene::IScene* pScene          : 
// @param tFrameResource& frame          : 
// @param tIndicesPool& ip               : 
// @param tVerticesPool& vp              : 
//
// @return ZRESULT  : 
//-------------------------------------------------------
ZRESULT CMafiaII::createScene(scene::IScene* pScene, tFrameNameTable& frameNameTable, 
                              tFrameResource& frame, tIndicesPool& ip, tVerticesPool& vp)
{
  //
  // create the root dummy node first:
  ZPtr<scene::INode>  pRoot;
  pScene->getRoot(&pRoot);
  ZMatrix mat;
  ZPtr<scene::IDummyNode> pDummy;
  createDummyNode(pRoot, m_strFileName, true, 1, 0x800000, ZVector(0.05f, 0.05f, 0.05f), mat, &pDummy);
  pRoot = NULL;
  pDummy->QueryInterface(IID_(scene::INode), (void**)&pRoot);
  pDummy = NULL;

  //
  // ok, create the rest;
  if (frame.m_bGrouping)
    for (long nGrp = 0; nGrp < frame.m_nRootGroups; nGrp++)
    {
      //
      // create root group:
      pDummy = NULL;
      createDummyNode(pRoot, frame.m_grouping.m_rootGroops[nGrp].m_strName, 
                      true, 1, 0x800000, ZVector(0.05f, 0.05f, 0.05f), mat, &pDummy);
    }
  else
    for (long nSceneSet = 0; nSceneSet < frame.m_nSceneCompositions; nSceneSet++)
    {
      //
      // create scene group:
      pDummy = NULL;
      createDummyNode(pRoot, &frameNameTable.m_strPool[frameNameTable.m_names[nSceneSet].wFirst], 
                      true, 1, 0x800000, ZVector(0.05f, 0.05f, 0.05f), mat, &pDummy);

    }

  /*

  //
  // lets try to visualize vertices buffer;
  ZPtr<scene::INode>      pNode;
  ZPtr<scene::IMeshNode>  pMNode;
  ZPtr<scene::IPolyMesh>  pMesh;
  ZMatrix mat;
  createMeshNode(m_pScene, &pNode, &pMNode, &pMesh);
  // drop vertices;
  struct tBigVert
  {
    WORD  x,y,z;  //something is wrong with Z?
    char  n[6];   //0? 1?, x, -z, y, 5?
    DWORD c;
    DWORD d;
    DWORD e;
    DWORD f;
    WORD  uv0[2];
    WORD  uv1[2];
    WORD  uv2[2];
  };
  tVerticesBuffer& vb = VerticesPool.m_Buffers[0];
  tIndicesBuffer&  ib = IndicesPool.m_Buffers[0];
  ZVertexBuf* pVBuf = new ZVertexBuf(VERTEX_HAS_POSITION | VERTEX_HAS_NORMAL | VERTEX_HAS_2UV, vb.m_nSize/sizeof(tBigVert));
  ZPolyBuf* pPBuf = new ZPolyBuf(ZPoly::TRI, ib.m_indices.length()/3);
  tBigVert* pVerts = reinterpret_cast<tBigVert*>(vb.m_vertices.getBuffer());
  for (long nV = 0; nV < pVBuf->length(); nV++)
  {
    ZVertex& v = (*pVBuf)[nV];
    v.position().x = word2float(pVerts[nV].x);
    v.position().z =-word2float(pVerts[nV].y);
    v.position().y = word2float(pVerts[nV].z, true);

    v.normal().x = byte2float(pVerts[nV].n[2]);
    v.normal().z =-byte2float(pVerts[nV].n[3]);
    v.normal().y = byte2float(pVerts[nV].n[4]);

    v.u() = float16to32(pVerts[nV].uv0[0]);
    v.v() = float16to32(pVerts[nV].uv0[1]);
  }
  tUShortSeq seq;
  for (long nP = 0; nP < pPBuf->length(); nP++)
  {
    seq[0] = ib.m_indices[nP*3+0];
    seq[1] = ib.m_indices[nP*3+1];
    seq[2] = ib.m_indices[nP*3+2];
    (*pPBuf)[nP].setExterior(seq);
  }
  pMesh->setVertexBuf(pVBuf);
  pMesh->setPolyBuf(pPBuf);
  pNode->changed(NC_GEOMETRY);
  attachSceneObject(pNode, NULL, "test", mat);

  */
  return ZRESULT_OK;
}


};//io

};//core
