/***************************************************************************
 *                          Zanoza Modeler v3.x                            *
 *      This is unpublished propietary source code of Zanoza Software.     *
 *      The copyright notice does not evidence any actual or intended      *
 *                    publication of such source code.                     *
 *                                                                         *
 *                Copyright (c) 2002-2012 Zanoza Software.                 *
 *                                                                         *
 *                        All rights reserved.                             *
 ***************************************************************************
 *--------------------------------------------------------------------------
 * @author:   Oleg M.
 * @author:   Torus Knot Software Ltd; Inspired by ORGE source code;
 *--------------------------------------------------------------------------
 * @purpose:  float16 routines
 *--------------------------------------------------------------------------
 * @histroy: 04.07.2008. Created
 */
#ifndef float16_h_included
#define float16_h_included
#include <zmodeler.h>

static inline DWORD float16to32I(WORD y)
{
  register int s = (y >> 15) & 0x00000001;
  register int e = (y >> 10) & 0x0000001f;
  register int m =  y        & 0x000003ff;

  if (e == 0)
  {
    if (m == 0) // Plus or minus zero
      return s << 31;
    else // Denormalized number -- renormalize it
    {
      while (!(m & 0x00000400))
      {
        m <<= 1;
        e -=  1;
      }
      e += 1;
      m &= ~0x00000400;
    }
  }
  else if (e == 31)
  {
    if (m == 0) // Inf
      return (s << 31) | 0x7f800000;
    else // NaN
      return (s << 31) | 0x7f800000 | (m << 13);
  }
  e = e + (127 - 15);
  m = m << 13;
  return (s << 31) | (e << 23) | m;
}

static inline float float16to32(WORD y)
{
  union 
  { 
    float  f; 
    DWORD  i; 
  }v;
  v.i = float16to32I(y);
  return v.f;
}

static inline WORD float32to16I(DWORD i)
{
  register int s =  (i >> 16) & 0x00008000;
  register int e = ((i >> 23) & 0x000000ff) - (127 - 15);
  register int m =   i        & 0x007fffff;

  if (e <= 0)
  {
    if (e < -10)
      return 0;
    m = (m | 0x00800000) >> (1 - e);
    return s | (m >> 13);
  }
  else if (e == 0xff - (127 - 15))
  {
    if (m == 0) // Inf
      return s | 0x7c00;
    else    // NAN
      m >>= 13;
    return s | 0x7c00 | m | (m == 0);
  }
  else
  {
    if (e > 30) // Overflow
      return s | 0x7c00;
    return s | (e << 10) | (m >> 13);
  }
}

static inline WORD float32to16(float i)
{
  union 
  { 
    float f; 
    DWORD i; 
  } v;
  v.f = i;
  return float32to16I(v.i);
}

#endif//float16_h_included
